#pragma once
#include <ReArchive/types/FileTable.h>
#include <filesystem>
#include <vector>

// TODO compression.
namespace ReArchive {
    /// Creates a new empty archive.
    /// @param filesystem_path where the archive is to be created.
    /// @param use_compression whether you'd like the file to use compression.
    /// @param running_tally If you're keeping track of the file table in your program, Providing it here will update it to reflect our changes.
    /// @note use_compression currently does nothing.
    /// @returns True if success.
   [[nodiscard]] bool CreateArchive(const std::filesystem::path& archive, bool use_compression = false, FileTable* running_tally = nullptr);

    /// Add a file to an archive.
    /// @param archive The archive on the disk.
    /// @param file_data The raw data of the file to be written.
    /// @param file_path The std::filesystem::path you would use to retrieve the file from the archive.
    /// @param byte_count The length of the file in bytes.
    /// @param running_tally If you're keeping track of the file table in your program, Providing it here will update it to reflect our changes.
    /// @returns True if success.
    [[nodiscard]] bool WriteFile(const std::filesystem::path& archive, const std::filesystem::path& file_path, const unsigned char* file_data, const int64_t& byte_count, FileTable* running_tally = nullptr);

    /// Overwrite a file which already exists in the archive.
    /// @param archive The archive on the disk.
    /// @param file_data The raw data of the file to be written.
    /// @param file_path The std::filesystem::path file in the archive to be overwritten.
    /// @param byte_count The length of the file in bytes.
    /// @note It is expected that byte_count will the the same as the file size.
    /// @returns True if success.
    [[nodiscard]] bool OverwriteFile(const std::filesystem::path& archive, const std::filesystem::path& file_path, const unsigned char* file_data, const int64_t& byte_count);

    /// Remove a file from the archive.
    /// @param archive The archive on the disk.
    /// @param file_path The std::filesystem::path file in the archive to be removed.
    /// @param running_tally If you're keeping track of the file table in your program, Providing it here will update it to reflect our changes.
    /// @returns True if success.
    [[nodiscard]] bool EraseFile(const std::filesystem::path& archive, const std::filesystem::path& file_path, FileTable* running_tally = nullptr);

    /// Read a file from a given archive
    /// @param archive The archive on the disk.
    /// @param file_path The std::filesystem::path you specified for the given file.
    /// @note An empty vector is returned in the event that no such file exists or there was an error reading it back.
    std::vector<unsigned char> ReadFile(const std::filesystem::path& archive, const std::filesystem::path& file_path);
}